<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id$
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

/**
 * Debugging library used to catch PHP warnings and write messages.
 *
 * It uses CSS styles: info, error, console, php_backtrace, php_error
 */

require_once('nuphp/http.php');
require_once('nuphp/session.php');

$log_console='';
$log_errors = Array();

function readDebugLevel()
{
    global $debug_nuface, $debug_nuface_verbose;

    $debug_level = getHttp('debug_level');
    if (!preg_match('/^[0-9]+$/', $debug_level))
        $debug_level = null;
    if (!is_null($debug_level)) {
        $debug_level = (int)$debug_level;
        $debug_level = max(min($debug_level, 2), 0);
        $_SESSION['debug_level'] = $debug_level;
    } else {
        $debug_level = getSession('debug_level', false);
    }

    if (is_null($debug_level))
        return;

    if (2 <= $debug_level) {
        $debug_nuface = true;
        $debug_nuface_verbose = true;
    } else if (1 <= $debug_level) {
        $debug_nuface = true;
        $debug_nuface_verbose = false;
    } else {
        $debug_nuface = false;
        $debug_nuface_verbose = false;
    }
}

function log_and_exit($exit_code){
    end_page();
    exit($exit_code);
}

function add_log_html($html, $show_backtrace=true)
{
    global $log_console, $debug_nuface_verbose;
    $log_console .= $html."\n";
    if ($show_backtrace and $debug_nuface_verbose) {
        if (is_array($show_backtrace))
            $log_console .= format_backtrace($show_backtrace);
        else
            $log_console .= php_backtrace();
    }
}

function add_log($text, $show_backtrace=true)
{
    add_log_html('<p class="info"><img src="images/info.png" alt="[Information]"> '.$text."</p>", $show_backtrace);
}

function log_error($html, $show_backtrace=true)
{
    global $log_errors;
    $log_errors[] = htmlspecialchars_decode($html);
    $html = '<p class="error"><img src="images/error.png" alt="[Error]"> '.$html."</p>";
    add_log_html($html, $show_backtrace);
}

function add_log_pre($text)
{
    add_log_html('<pre>'.htmlspecialchars($text).'</pre>');
}

function escape_javascript($text)
{
    $text = str_replace('\\', '\\\\', $text);
    $text = str_replace("'", '\\\'', $text);
    return $text;
}

function quote_javascript($text)
{
    $text = escape_javascript($text);
    return "'$text'";
}


function print_console_content()
{
    global $log_console, $log_errors;
    if ($log_console) {
        print "<div class=\"console\">$log_console</div>\n";
    }
    $log_console = '';
    $max_errors = 4;
    if ($log_errors) {
        $truncated = false;
        if ($max_errors < count($log_errors)) {
            $log_errors = array_slice($log_errors, 0, $max_errors);
            $log_errors[] = '(...)';
        }
        $log_errors = array_map('escape_javascript', $log_errors);
        $message = implode($log_errors, '\n\n');
        echo '<script type="text/javascript">alert(\''.$message.'\');</script>';
    }
    $log_errors = Array();
}

function print_pre($r, $name=null)
{
    print '<pre>';
    if ($name)
        echo $name.' = ';
    $type = gettype($r);
    if (($type == 'string') or ($type == 'boolean') or ($type == 'NULL')) {
        var_dump($r);
    } else {
        print_r($r);
    }
    print '</pre>';
}

function getmicrotime(){
    list($usec, $sec) = explode(" ",microtime());
    return ((float)$usec + (float)$sec);
}

$debug_rootdir = realpath('.').'/';
function relative_filename($filename)
{
    global $debug_rootdir;
    return ereg_replace($debug_rootdir, './',  $filename);
}

// Format backtrace as HTML string
function format_backtrace($backtrace, $skip=0)
{
    if (count($backtrace)-$skip <= 0)
        return '';

    $txt  = "<div class=\"php_backtrace\"><p>Backtrace :</p>\n";
    $txt .= "<ol>\n";
    for($index=$skip; $index<count($backtrace); $index++)
    {
        $call = $backtrace[$index];
        $txt .= "<li>";
        $func = $call['function'];
        if (array_key_exists('class', $call))
            $func = $call['class'].'::'.$func;
        $txt .= '<em>'.$func.'</em>';
        if (array_key_exists('file', $call))
        {
            $file = relative_filename($call['file']);
            $txt .= ' at <em>'.$file.':'.$call['line'].'</em>';
        }
        $txt .= "</li>\n";
    }
    $txt .= "</ol></div>\n";
    return $txt;
}

// Get current backtrace
// $skip is the number of functions to skip
function php_backtrace($skip=1)
{
    if (!function_exists('debug_backtrace'))
        return '';
    $backtrace = debug_backtrace();
    return format_backtrace($backtrace, $skip+1);
}


function nuface_error_handler($errno, $errstr, $errfile, $errline)
{
    global $debug_nuface, $debug_nuface_verbose;

    // In release mode, ignore minor errors
    if (!$debug_nuface &&
        (($errno == E_NOTICE) || ($errno == E_WARNING)
         || ($errno == E_STRICT)))
    {
        return true;
    }

    // Ignore PHP5 strict errors
    if ($errno == E_STRICT) {
        return true;
    }

    if ($debug_nuface_verbose and function_exists('debug_backtrace'))
        $backtrace = debug_backtrace();
    else
        $backtrace = null;

    switch($errno)
    {
    case E_NOTICE: $errname = _('Notice'); break;
    case E_STRICT: $errname = _('Strict notice'); break;
    case E_WARNING: $errname = _('Warning'); break;
    case E_PARSE: $errname = _('Syntax error'); break;
    case E_ERROR: $errname = _('Fatal error'); break;
    default: $errname = _("Error"); break;
    }

    return nuface_full_error_handler($errname, $errstr, $errfile, $errline, $backtrace, 1);
}

function nuface_full_error_handler($errname, $errstr, $errfile, $errline, $backtrace, $backtrace_skip=0)
{
    global $debug_nuface, $debug_nuface_verbose, $log_console;

    // Error type
    $txt = '<div class="php_error"><p class="error"><img src="images/error.png" alt="'._('Error').'"> ';

    // Add error origin and error message
    if ($debug_nuface)
    {
        $file = relative_filename($errfile);
        $txt .= sprintf(_('%s in %s: '),
            $errname, '<em>'.$file.':'.$errline.'</em>');
    } else {
        $txt .= sprintf(_('%s: '),
            $errname);
    }
    $txt .= $errstr;

    // Add backtrace
    $txt .= '</p>';
    if ($debug_nuface_verbose and $backtrace)
        $txt .= format_backtrace($backtrace, $backtrace_skip);
    $txt .= '</div>'."\n";

    // Affiche l'erreur (la methode depend du type d'erreur)
    $log_console .= $txt;

    /* Don't execute PHP internal handler  */
    return true;
}

// Configuration du gestionnaire d'erreurs
set_error_handler("nuface_error_handler");

register_shutdown_function('print_console_content');

if (0 <= version_compare(phpversion(), '5')) {
    require_once('nuphp/debug_php5.php');
}

?>
