<?php
# Copyright(C) 2004-2005 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: common.php 9724 2007-09-25 12:50:23Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('nuphp/http.php');    # getHttp()
require_once('nuphp/session.php'); # getSession()
require_once('nuphp/debug.php');   # log_error()

# Language identifier (2 letters) => C locale (string or array of strings)
$AVAILABLE_LANGUAGES = array(
    'fr' => Array("fr_FR.UTF-8", "fr_FR", "fr"),
    'en' => Array("en_US.UTF-8", "en_US", "en_US.UTF-8", "en_US", "en"),
);

function get_browser_language($valid_languages, $default=null)
{
    # Read HTTP_ACCEPT_LANGUAGE browser variable
    if (!array_key_exists("HTTP_ACCEPT_LANGUAGE", $_SERVER)) {
        return $default;
    }
    $http_accept = explode(",",$_SERVER["HTTP_ACCEPT_LANGUAGE"]);

    # Parse each language to get language weight
    $languages = Array();
    foreach ($http_accept as $index=>$lang)
    {
        if (ereg('([^;]+);q=([0-9]+\.[0-9]+)$', $lang, $regs)) {
            $lang = $regs[1];
            $key = (float)$regs[2];
        } else {
            $key = 1.0 - (float)$index * 0.01;
        }
        $languages[$lang] = $key;
    }

    # Find the language with the higher weight
    arsort($languages);
    foreach ($languages as $language=>$weight)
    {
        $language = substr($language, 0, 2);
        if (in_array($language, $valid_languages)) {
            return $language;
        }
    }

    # No matching language: fallback to default
    return $default;
}

function init_i18n($default_language)
{
    global $AVAILABLE_LANGUAGES;

    # Choose language
    $lang = get_http_language();
    if (!$lang) {
        # Read session variable ("lang")
        $lang = get_language();
    }
    if (!$lang) {
        # Guess browser language
        $lang = get_browser_language(array_keys($AVAILABLE_LANGUAGES));
    }
    if (!$lang) {
        $lang = $default_language;
    }

    # Set language variables and environment
    $locale = $AVAILABLE_LANGUAGES[$lang];
    $ok = setlocale(LC_ALL, $locale);
    if ($ok === false) {
        log_error(sprintf('Warning: Unable to setup locale, needed for messages translation (locale: "%s").',
            implode("; ", $locale)));
    }
    putenv("LANGUAGE=$lang");
    $_SESSION['lang'] = $lang;
}

function init_gettext($domain, $locale_dir)
{
    bindtextdomain($domain, $locale_dir);
    bind_textdomain_codeset($domain, 'utf8');
    textdomain($domain);
}

function get_language()
{
    return getSession('lang', false);
}

# Update current language if needed
function get_http_language()
{
    global $AVAILABLE_LANGUAGES;
    $lang = getHttp('lang');
    if (!$lang)
        return null;
    if (!array_key_exists($lang, $AVAILABLE_LANGUAGES))
        return null;
    return $lang;
}

?>
